import React from "react";
import { Table } from "react-bootstrap";
import CollapsibleBox from "compendium/components/CollapsibleBox";

const TICK_ICON = <span>&#10004;</span>;

const CELL_SIZE = 8

interface ScrollableMatrixProps {

    // dataLookup is a map of NRENs, years, and categories that maps to data for that category in that year for that NREN
    dataLookup: Map<string, Map<number, Map<string, { [key: string]: { [key: string]: string | number } }>>>;

    // rowInfo is a map of row titles and the lookup key that maps to data for that row
    rowInfo: { [key: string]: string };

    // categoryLookup is a map of category titles and the lookup key that maps to data for that category
    categoryLookup: { [key: string]: string };

    isTickIcon?: boolean;
}

export function ScrollableMatrix({ dataLookup, rowInfo, categoryLookup, isTickIcon = false }: ScrollableMatrixProps): React.ReactElement {

    if (!dataLookup) {
        return <div className="matrix-border"></div>
    }

    const categoryContainers = Object.entries(categoryLookup).map(([categoryKey, categoryTitle]) => {
        const rows = Object.entries(rowInfo).map(([title, field]) => {

            const dataForRow: (string | number | React.JSX.Element | null)[] = [];

            Array.from(dataLookup.entries()).sort(
                ([nrenA], [nrenB]) => nrenA.localeCompare(nrenB)
            ).forEach(([_nren, nrenData]) => {
                nrenData.forEach((yearData) => {
                    const valuesForCategory = yearData.get(categoryKey);
                    if (!valuesForCategory) return;

                    const fieldValue = valuesForCategory[field] != null ? Object.keys(valuesForCategory[field])[0] : null;

                    const value = fieldValue != null && isTickIcon ? TICK_ICON : fieldValue;
                    dataForRow.push(value);
                });
            });

            if (!dataForRow.length) {
                // don't show the row if there's no data
                return null;
            }
            return (<tr key={title}>
                <th className="fixed-column">{title}</th>
                {dataForRow.map((value, index) => (
                    <td key={index}>{value}</td>
                ))}
            </tr>)
        });

        const nrenYears: { [key: string]: number[] } = Array.from(dataLookup.entries()).sort(
            ([nrenA], [nrenB]) => nrenA.localeCompare(nrenB)
        ).reduce((nrenYears, [nren, nrenData]) => {

            Array.from(nrenData.entries()).forEach(([year, yearData]) => {
                const has_category = yearData.get(categoryKey)
                if (has_category) {
                    if (!nrenYears[nren]) {
                        nrenYears[nren] = []
                    }
                    nrenYears[nren].push(year)
                }
            })

            return nrenYears;
        }, {});

        for (const nren in nrenYears) {
            nrenYears[nren].sort((a, b) => a - b)
        }

        const categoryContent = (
            <CollapsibleBox title={categoryTitle} startCollapsed key={categoryKey} theme="-matrix">
                {
                    !rows ? <div style={{ paddingLeft: "5%" }}><p>No data available for this section.</p></div> :
                        <div className="table-responsive">
                            <Table className="matrix-table" bordered>
                                <thead>
                                    {(() => {
                                        const nrenEntries = Object.entries(nrenYears);
                                        return (
                                            <>
                                                <tr>
                                                    <th className="fixed-column"></th>
                                                    {nrenEntries.map(([nren, years]) => (
                                                        <th key={nren} colSpan={years.length} style={{ width: `${years.length * CELL_SIZE}rem` }}>
                                                            {nren}
                                                        </th>
                                                    ))}
                                                </tr>
                                                <tr>
                                                    <th className="fixed-column"></th>
                                                    {nrenEntries.flatMap(([nren, years]) =>
                                                        years.map((year) => (
                                                            <th key={`${nren}-${year}`}>{year}</th>
                                                        ))
                                                    )}
                                                </tr>
                                            </>
                                        );
                                    })()}
                                </thead>
                                <tbody>
                                    {rows}
                                </tbody>
                            </Table>
                        </div>
                }

            </CollapsibleBox>
        );
        return categoryContent;
    })

    return <div className="matrix-border">
        {categoryContainers}
    </div>;
}