import React, { useContext, useState, useRef, useEffect } from 'react';
import * as htmlToImage from 'html-to-image';
import { ImageType } from "compendium/helpers/constants";
import { ChartContainerContext } from "compendium/providers/ChartContainerProvider";
import { FaDownload } from 'react-icons/fa';

interface DownloadImageChartProps {
    filename: string;
}

const DownloadImageChartButton: React.FC<DownloadImageChartProps> = ({ filename }) => {
    const chartContainerRef = useContext(ChartContainerContext);
    const [isOpen, setIsOpen] = useState(false);
    const dropdownRef = useRef(null);

    const downloadChartAsImage = async (exportType) => {
        if (chartContainerRef?.current) {
            setIsOpen(false);
            const scale = 1;

            const commonStyle = {
                transform: `scale(${scale})`,
                'transform-origin': 'top left',
                background: 'white'
            };

            let dataUrl;
            switch (exportType) {
                case ImageType.JPEG:
                    dataUrl = await htmlToImage.toJpeg(chartContainerRef.current, {
                        quality: 0.95,
                        style: commonStyle
                    });
                    break;
                case ImageType.SVG:
                    dataUrl = await htmlToImage.toSvg(chartContainerRef.current, {
                        style: commonStyle
                    });
                    break;
                case ImageType.PNG:
                default:
                    dataUrl = await htmlToImage.toPng(chartContainerRef.current, {
                        style: commonStyle
                    });
                    break;
            }

            const link = document.createElement('a');
            link.href = typeof dataUrl === 'string' ? dataUrl : URL.createObjectURL(dataUrl);
            link.download = `${filename}.${exportType}`;

            document.body.appendChild(link);
            link.click();
            document.body.removeChild(link);
        }
    }

    const toggleDropdown = () => {
        setIsOpen(!isOpen);
    };

    const handleClickOutside = (event) => {

        if (dropdownRef.current && !(dropdownRef.current as HTMLElement).contains(event.target)) {
            setIsOpen(false);
        }
    };

    useEffect(() => {
        document.addEventListener('mousedown', handleClickOutside);

        return () => {
            document.removeEventListener('mousedown', handleClickOutside);
        };
    }, []);

    return (
        <div className={`image-dropdown`} ref={dropdownRef}>
            <button className={`downloadbutton downloadimage`} onClick={toggleDropdown}>
                IMAGE <FaDownload />
            </button>
            {isOpen && <div className={`image-options`}>
                <div className={`imageoption downloadpng`} onClick={() => downloadChartAsImage(ImageType.PNG)}>
                    <span>PNG</span>
                </div>
                <div className={`imageoption downloadjpeg`} onClick={() => downloadChartAsImage(ImageType.JPEG)}>
                    <span>JPEG</span>
                </div>
                <div className={`imageoption downloadsvg`} onClick={() => downloadChartAsImage(ImageType.SVG)}>
                    <span>SVG</span>
                </div>
            </div>}
        </div>
    );
}

export default DownloadImageChartButton;
