import { Chart as ChartJS } from 'chart.js';

interface options {
    title?: string;
    unit?: string;
    tooltipPrefix?: string;
    tooltipUnit?: string;
    tickLimit?: number;
    valueTransform?: (value) => number | string;
}
export const getLineChartOptions = ({ title, unit, tooltipPrefix, tooltipUnit, tickLimit, valueTransform }: options) => {
    return {
        responsive: true,
        elements: {
            point: {
                pointStyle: 'circle',
                pointRadius: 4,
                pointBorderWidth: 2,
                pointBackgroundColor: 'white',
            },
        },
        animation: {
            duration: 0
        },
        plugins: {
            legend: {
                display: false,
            },
            tooltip: {
                callbacks: {
                    label: function (tooltipItem) {
                        let label = tooltipPrefix ?? (tooltipItem.dataset.label || '');
                        const value = valueTransform ? valueTransform(tooltipItem.parsed.y) : tooltipItem.parsed.y;

                        if (tooltipItem.parsed.y !== null) {
                            label += `: ${value} ${tooltipUnit || ''}`
                        }
                        return label;
                    }
                },
            },
        },
        scales: {
            y: {
                title: {
                    display: !!title,
                    text: title || '',
                },
                ticks: {
                    autoSkip: true,
                    maxTicksLimit: tickLimit,
                    callback: (value: string | number) => {

                        const label = typeof value === 'string' ? value : valueTransform ? valueTransform(value) : value;
                        return `${label} ${unit || ''}`;
                    }
                },
            },
        },
    };

}

export const getBarChartOptions = ({ title, unit, tooltipPrefix, tooltipUnit, valueTransform }: options) => {
    return {
        maintainAspectRatio: false,
        layout: {
            padding: {
                right: 60
            }
        },
        animation: {
            duration: 0
        },
        plugins: {
            legend: {
                display: false,
            },
            chartDataLabels: {
                font: {
                    family: `"Open Sans", sans-serif`
                },
            },
            tooltip: {
                callbacks: {
                    label: function (tooltipItem) {
                        let label = tooltipPrefix ?? (tooltipItem.dataset.label || '');
                        const value = valueTransform ? valueTransform(tooltipItem.parsed.x) : tooltipItem.parsed.x;

                        if (tooltipItem.parsed.y !== null) {
                            label += `: ${value} ${tooltipUnit || ''}`
                        }
                        return label;
                    }
                },
            },
        },
        scales: {
            x: {
                title: {
                    display: !!title,
                    text: title || '',
                },
                position: 'top' as const,
                ticks: {
                    callback: (value: string | number) => {
                        if (!value) return value;
                        const val = valueTransform ? valueTransform(value) : value;
                        return `${val} ${unit || ''}`;
                    },
                },
            },
            x2: {
                title: {
                    display: !!title,
                    text: title || '',
                },
                ticks: {
                    callback: (value: string | number) => {
                        if (!value) return value;
                        const val = valueTransform ? valueTransform(value) : value;
                        return `${val} ${unit || ''}`;
                    },
                },
                grid: {
                    drawOnChartArea: false
                },
                afterDataLimits: function (axis) {
                    const indices = Object.keys(ChartJS.instances)

                    // initial values should be far outside possible range
                    let max = -999999
                    let min = 999999

                    for (const index of indices) {
                        if (ChartJS.instances[index] && axis.chart.scales.x2) {
                            min = Math.min(ChartJS.instances[index].scales.x.min, min);
                            max = Math.max(ChartJS.instances[index].scales.x.max, max);
                        }
                    }

                    axis.chart.scales.x2.options.min = min;
                    axis.chart.scales.x2.options.max = max;
                    axis.chart.scales.x2.min = min;
                    axis.chart.scales.x2.max = max;
                },
            },
            y: {
                stacked: true,
                ticks: {
                    autoSkip: false,
                },
            },
        },
        indexAxis: "y" as const
    };
}