import { TRACK_TYPES } from 'compendium/matomo/constants'
import {
    CustomDimension,
    TrackEventParams,
    TrackLinkParams,
    TrackPageViewParams,
    TrackParams,
    UserOptions,
} from 'compendium/matomo/types'

class MatomoTracker {
    mutationObserver?: MutationObserver

    constructor(userOptions: UserOptions) {
        if (!userOptions.urlBase) {
            throw new Error('Matomo urlBase is required.')
        }
        if (!userOptions.siteId) {
            throw new Error('Matomo siteId is required.')
        }

        this.initialize(userOptions)
    }

    private initialize({
        urlBase,
        siteId,
        userId,
        trackerUrl,
        srcUrl,
        disabled,
        heartBeat,
        requireConsent = false,
        configurations = {},
    }: UserOptions) {
        const normalizedUrlBase =
            urlBase[urlBase.length - 1] !== '/' ? `${urlBase}/` : urlBase

        if (typeof window === 'undefined') {
            return
        }

        // @ts-expect-error - _paq is defined in the Matomo script
        window._paq = window._paq || []

        // @ts-expect-error - _paq is defined in the Matomo script
        if (window._paq.length !== 0) {
            return
        }

        if (disabled) {
            return
        }

        if (requireConsent) {
            this.pushInstruction('requireConsent')
        }

        this.pushInstruction(
            'setTrackerUrl',
            trackerUrl ?? `${normalizedUrlBase}matomo.php`,
        )

        this.pushInstruction('setSiteId', siteId)

        if (userId) {
            this.pushInstruction('setUserId', userId)
        }

        Object.entries(configurations).forEach(([name, instructions]) => {
            if (instructions instanceof Array) {
                this.pushInstruction(name, ...instructions)
            } else {
                this.pushInstruction(name, instructions)
            }
        })

        // accurately measure the time spent on the last pageview of a visit
        if (!heartBeat || (heartBeat && heartBeat.active)) {
            this.enableHeartBeatTimer((heartBeat && heartBeat.seconds) ?? 15)
        }

        const doc = document
        const scriptElement = doc.createElement('script')
        const scripts = doc.getElementsByTagName('script')[0]

        scriptElement.type = 'text/javascript'
        scriptElement.async = true
        scriptElement.defer = true
        scriptElement.src = srcUrl || `${normalizedUrlBase}matomo.js`

        if (scripts && scripts.parentNode) {
            scripts.parentNode.insertBefore(scriptElement, scripts)
        }
    }

    enableHeartBeatTimer(seconds: number): void {
        this.pushInstruction('enableHeartBeatTimer', seconds)
    }

    private trackEventsForElements(elements: HTMLElement[]) {
        if (elements.length) {
            elements.forEach((element) => {
                element.addEventListener('click', () => {
                    const { matomoCategory, matomoAction, matomoName, matomoValue } =
                        element.dataset
                    if (matomoCategory && matomoAction) {
                        this.trackEvent({
                            category: matomoCategory,
                            action: matomoAction,
                            name: matomoName,
                            value: Number(matomoValue),
                        })
                    } else {
                        throw new Error(
                            `Error: data-matomo-category and data-matomo-action are required.`,
                        )
                    }
                })
            })
        }
    }

    // Tracks events based on data attributes
    trackEvents(): void {
        const matchString = '[data-matomo-event="click"]'
        let firstTime = false
        if (!this.mutationObserver) {
            firstTime = true
            this.mutationObserver = new MutationObserver((mutations) => {
                mutations.forEach((mutation) => {
                    mutation.addedNodes.forEach((node) => {
                        // only track HTML elements
                        if (!(node instanceof HTMLElement)) return

                        // check the inserted element for being a code snippet
                        if (node.matches(matchString)) {
                            this.trackEventsForElements([node])
                        }

                        const elements = Array.from(
                            node.querySelectorAll<HTMLElement>(matchString),
                        )
                        this.trackEventsForElements(elements)
                    })
                })
            })
        }
        this.mutationObserver.observe(document, { childList: true, subtree: true })

        // Now track all already existing elements
        if (firstTime) {
            const elements = Array.from(
                document.querySelectorAll<HTMLElement>(matchString),
            )
            this.trackEventsForElements(elements)
        }
    }

    stopObserving(): void {
        if (this.mutationObserver) {
            this.mutationObserver.disconnect()
        }
    }

    // Tracks events
    // https://matomo.org/docs/event-tracking/#tracking-events
    trackEvent({
        category,
        action,
        name,
        value,
        ...otherParams
    }: TrackEventParams): void {
        if (category && action) {
            this.track({
                data: [TRACK_TYPES.TRACK_EVENT, category, action, name, value],
                ...otherParams,
            })
        } else {
            throw new Error(`Error: category and action are required.`)
        }
    }

    // Gives consent for tracking, this is required for Matomo to start tracking when requireConsent is set to true
    // https://developer.matomo.org/guides/tracking-consent
    giveConsent(): void {
        this.pushInstruction('setConsentGiven')
    }

    // Tracks outgoing links to other sites and downloads
    // https://developer.matomo.org/guides/tracking-javascript-guide#enabling-download-outlink-tracking
    trackLink({ href, linkType = 'link' }: TrackLinkParams): void {
        this.pushInstruction(TRACK_TYPES.TRACK_LINK, href, linkType)
    }

    // Tracks page views
    // https://developer.matomo.org/guides/spa-tracking#tracking-a-new-page-view
    trackPageView(params?: TrackPageViewParams): void {
        this.track({ data: [TRACK_TYPES.TRACK_VIEW], ...params })
    }

    // Sends the tracked page/view/search to Matomo
    track({
        data = [],
        documentTitle = window.document.title,
        href,
        customDimensions = false,
    }: TrackParams): void {
        if (data.length) {
            if (
                customDimensions &&
                Array.isArray(customDimensions) &&
                customDimensions.length
            ) {
                customDimensions.map((customDimension: CustomDimension) =>
                    this.pushInstruction(
                        'setCustomDimension',
                        customDimension.id,
                        customDimension.value,
                    ),
                )
            }

            this.pushInstruction('setCustomUrl', href ?? window.location.href)
            this.pushInstruction('setDocumentTitle', documentTitle)
            this.pushInstruction(...(data as [string, ...any[]])) // eslint-disable-line @typescript-eslint/no-explicit-any
        }
    }

    /**
     * Pushes an instruction to Matomo for execution, this is equivalent to pushing entries into the `_paq` array.
     *
     * For example:
     *
     * ```ts
     * pushInstruction('setDocumentTitle', document.title)
     * ```
     * Is the equivalent of:
     *
     * ```ts
     * _paq.push(['setDocumentTitle', document.title]);
     * ```
     *
     * @param name The name of the instruction to be executed.
     * @param args The arguments to pass along with the instruction.
     */
    pushInstruction(name: string, ...args: any[]): MatomoTracker { // eslint-disable-line @typescript-eslint/no-explicit-any
        if (typeof window !== 'undefined') {
            // @ts-expect-error - _paq is defined in the Matomo script
            window._paq.push([name, ...args])
        }

        return this
    }
}

export function createInstance(params: UserOptions): MatomoTracker {
    // if on localhost or not production, 
    // disable Matomo tracking
    const isLocalhost = window.location.hostname === 'localhost'
    if (process.env.NODE_ENV !== 'production' || isLocalhost) {
        console.log('Matomo tracking disabled in development mode.')
        params.disabled = true
    }
    return new MatomoTracker(params)
}

export default MatomoTracker