import React, { useContext } from "react";
import { AlienWave } from "compendium/Schema";
import { createNrenTableLookup, addTooltip } from "compendium/helpers/dataconversion";
import DataPage from "compendium/components/DataPage";
import Filter from "compendium/components/graphing/Filter";
import { Sections } from "compendium/helpers/constants";
import { FilterSelectionContext } from "compendium/providers/FilterSelectionProvider";
import ChartContainer from "compendium/components/graphing/ChartContainer";
import { useData } from "compendium/helpers/useData";
import PillTable from "compendium/components/PillTable";
import titles from "compendium/titles";

function AlienWavePage(): React.ReactElement {
    const dataField = 'alien_wave_third_party';
    const validityCheck = (data: AlienWave) => data[dataField] !== null;
    const { filterSelection, setFilterSelection } = useContext(FilterSelectionContext);
    const { data, years, nrens } = useData<AlienWave>('/api/alien-wave', setFilterSelection, validityCheck);

    const selectedData = data.filter(data =>
        filterSelection.selectedYears.includes(data.year) && filterSelection.selectedNrens.includes(data.nren)
    );

    const dataLookup = createNrenTableLookup(selectedData, dataField);

    const processTooltip = (column: string, alienwave: AlienWave) => {
        if (!alienwave.nr_of_alien_wave_third_party_services) {
            return;
        }
        return `No. of alien wavelength services: ${alienwave.nr_of_alien_wave_third_party_services} `
    }

    const withTooltip = addTooltip(dataLookup, processTooltip);

    const columns = [
        "Yes",
        "Planned",
        "No",
    ]

    const columnValueMap = new Map<string, string>([
        [columns[0], "yes"],
        [columns[1], "planned"],
        [columns[2], "no"],
    ]);

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
        coloredYears
    />

    return (
        <DataPage title={titles["alien-wave"]}
            description="The table below shows NREN usage of alien wavelength or lightpath services provided by third parties. 
            It does not include alien waves used internally inside the NRENs own networks, as that is covered in another table. 
            In the optical network world, the term “alien wavelength” or “alien wave” (AW) is used to describe wavelengths in a 
            DWDM line system that pass through the network, i.e. they are not sourced/terminated by the line-system operator’s 
            equipment (hence “alien”). This setup is in contrast to traditional DWDM systems, where the DWDM light source 
            (transponder) operates in the same management domain as the amplifiers.

            Where NRENs have given the number of individual alien wavelength services, the figure is available in a hover-over 
            box. These are indicated by a black line around the coloured marker."
            category={Sections.Network} filter={filterNode}
            data={selectedData} filename="alien_wave_nrens_per_year">
            <ChartContainer>
                <PillTable columns={columns} columnLookup={columnValueMap} dataLookup={withTooltip} />
            </ChartContainer>
        </DataPage>
    );
}
export default AlienWavePage;
