import React, { useContext } from "react";
import { Table } from "react-bootstrap";

import { NetworkAutomation } from "compendium/Schema";
import { createNrenTableLookup } from "compendium/helpers/dataconversion";
import ColorPill from "compendium/components/ColorPill";
import DataPage from "compendium/components/DataPage";
import Filter from "compendium/components/graphing/Filter";
import { Sections } from "compendium/helpers/constants";
import { FilterSelectionContext } from "compendium/providers/FilterSelectionProvider";
import ChartContainer from "compendium/components/graphing/ChartContainer";
import { useData } from "compendium/helpers/useData";
import titles from "compendium/titles";

function AutomationPage(): React.ReactElement {
    const { filterSelection, setFilterSelection } = useContext(FilterSelectionContext);
    const { data: pertTeamData, years, nrens } = useData<NetworkAutomation>('/api/network-automation', setFilterSelection);

    const selectedData = (pertTeamData).filter(data =>
        filterSelection.selectedYears.includes(data.year) && filterSelection.selectedNrens.includes(data.nren)
    );

    const dataLookup = createNrenTableLookup(selectedData, 'network_automation');

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
        coloredYears
    />

    const showYears = [...filterSelection.selectedYears.filter(year => years.has(year))].sort();

    return (
        <DataPage title={titles["network-automation"]}
            description="The table below shows which NRENs have, or plan to, automate their 
            operational processes, with specification of which processes, and the names of 
            software and tools used for this given when appropriate. 
            Where NRENs indicated that they are using automation for some network tasks, 
            but did not specify which type of tasks, a marker has been placed in the 'other' column."
            category={Sections.Network} filter={filterNode}
            data={selectedData} filename="network_automation_nrens_per_year">
            <ChartContainer>
                <Table className="charging-struct-table" striped bordered >
                    <colgroup>
                        <col span={1} style={{ width: "16%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                        <col span={2} style={{ width: "12%" }} />
                    </colgroup>
                    <thead>
                        <tr>
                            <th></th>
                            <th colSpan={2}>Device Provisioning</th>
                            <th colSpan={2}>Data Collection</th>
                            <th colSpan={2}>Configuration Management</th>
                            <th colSpan={2}>Compliance</th>
                            <th colSpan={2}>Reporting</th>
                            <th colSpan={2}>Troubleshooting</th>
                            <th colSpan={2}>Other</th>
                        </tr>
                        <tr>
                            <th></th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                        </tr>
                    </thead>
                    <tbody>
                        {Array.from(dataLookup.entries()).map(([nren, nrenMap]) => (
                            <tr key={nren}>
                                <td>{nren}</td>
                                {["provisioning", "data_collection", "config_management", "compliance", "reporting", "troubleshooting"].map(column_key => (
                                    <>
                                        <td key={`${nren}-${column_key}-yes`}>
                                            {nrenMap.has("yes") &&
                                                showYears.map(year => {
                                                    const chargingYears = nrenMap.get("yes")?.get(year);
                                                    const serviceInfo = chargingYears ? chargingYears.network_automation_specifics : null;
                                                    return (
                                                        <ColorPill
                                                            key={year}
                                                            year={year}
                                                            active={!!(nrenMap.get("yes")?.has(year)) && !!(serviceInfo && serviceInfo.indexOf(column_key) > -1)}
                                                            tooltip={""}
                                                            rounded
                                                        />
                                                    );
                                                })
                                            }
                                        </td>
                                        <td key={`${nren}-${column_key}-planned`}>
                                            {nrenMap.has("planned") &&
                                                showYears.map(year => {
                                                    const chargingYears = nrenMap.get("planned")?.get(year);
                                                    const serviceInfo = chargingYears ? chargingYears.network_automation_specifics : null;
                                                    return (
                                                        <ColorPill
                                                            key={year}
                                                            year={year}
                                                            active={!!(nrenMap.get("planned")?.has(year)) && !!(serviceInfo && serviceInfo.indexOf(column_key) > -1)}
                                                            tooltip={""}
                                                            rounded
                                                        />
                                                    );
                                                })
                                            }
                                        </td>
                                    </>
                                ))}
                                <td key={`${nren}-other-yes`}>
                                    {nrenMap.has("yes") &&
                                        showYears.map(year => {
                                            const chargingYears = nrenMap.get("yes")?.get(year);
                                            const serviceInfo = chargingYears ? chargingYears.network_automation_specifics : null;
                                            return (
                                                <ColorPill
                                                    key={year}
                                                    year={year}
                                                    active={!!(nrenMap.get("yes")?.has(year)) && !!(serviceInfo && serviceInfo.length == 0)}
                                                    tooltip={""}
                                                    rounded
                                                />
                                            );
                                        })
                                    }
                                </td>
                                <td key={`${nren}-other-planned`}>
                                    {nrenMap.has("planned") &&
                                        showYears.map(year => {
                                            const chargingYears = nrenMap.get("planned")?.get(year);
                                            const serviceInfo = chargingYears ? chargingYears.network_automation_specifics : null;
                                            return (
                                                <ColorPill
                                                    key={year}
                                                    year={year}
                                                    active={!!(nrenMap.get("planned")?.has(year)) && !!(serviceInfo && serviceInfo.length == 0)}
                                                    tooltip={""}
                                                    rounded
                                                />
                                            );
                                        })
                                    }
                                </td>
                            </tr>
                        ))}

                    </tbody>
                </Table>
            </ChartContainer>
        </DataPage>
    );
}
export default AutomationPage;
