import React, { useContext } from "react";
import { Table } from "react-bootstrap";

import { NetworkFunctionVirtualisation } from "compendium/Schema";
import { createNrenTableLookup } from "compendium/helpers/dataconversion";
import ColorPill from "compendium/components/ColorPill";
import DataPage from "compendium/components/DataPage";
import Filter from "compendium/components/graphing/Filter";
import { Sections } from "compendium/helpers/constants";
import { FilterSelectionContext } from "compendium/providers/FilterSelectionProvider";
import ChartContainer from "compendium/components/graphing/ChartContainer";
import { useData } from "compendium/helpers/useData";
import titles from "compendium/titles";

function NetworkFunctionVirtualisationPage(): React.ReactElement {
    const { filterSelection, setFilterSelection } = useContext(FilterSelectionContext);
    const { data, years, nrens } = useData<NetworkFunctionVirtualisation>('/api/nfv', setFilterSelection);

    const selectedData = data.filter(data =>
        filterSelection.selectedYears.includes(data.year) && filterSelection.selectedNrens.includes(data.nren)
    );

    const dataLookup = createNrenTableLookup(selectedData, 'nfv_specifics');

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
        coloredYears
    />

    const showYears = [...filterSelection.selectedYears.filter(year => years.has(year))].sort();

    return (
        <DataPage title={titles.nfv}
            description="The table below shows the kinds of Network Function Virtualisation (NFV) used by NRENs."
            category={Sections.Network} filter={filterNode}
            data={selectedData} filename="network_function_virtualisation_nrens_per_year">
            <ChartContainer>
                <Table className="charging-struct-table" striped bordered >
                    <colgroup>
                        <col span={1} style={{ width: "20%" }} />
                        <col span={2} style={{ width: "16%" }} />
                        <col span={2} style={{ width: "16%" }} />
                        <col span={2} style={{ width: "16%" }} />
                        <col span={2} style={{ width: "16%" }} />
                        <col span={2} style={{ width: "16%" }} />
                    </colgroup>
                    <thead>
                        <tr>
                            <th></th>
                            <th colSpan={2}>Routers/switches</th>
                            <th colSpan={2}>Firewalls</th>
                            <th colSpan={2}>Load balancers</th>
                            <th colSpan={2}>VPN Concentrator Services</th>
                            <th colSpan={2}>Other</th>
                        </tr>
                        <tr>
                            <th></th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                            <th>Yes</th>
                            <th>Planned</th>
                        </tr>
                    </thead>
                    <tbody>
                        {Array.from(dataLookup.entries()).map(([nren, nrenMap]) => (
                            <tr key={nren}>
                                <td>{nren}</td>
                                {["routers", "firewalls", "load_balancers", "vpn_concentrators"].map(column_key => (
                                    <>
                                        <td key={`${column_key}-yes`}>
                                            {nrenMap.has(column_key) &&
                                                showYears.map(year => {
                                                    const chargingYears = nrenMap.get(column_key)!;
                                                    const serviceInfo = chargingYears.get(year);
                                                    return (
                                                        <ColorPill
                                                            key={year}
                                                            year={year}
                                                            active={chargingYears.has(year) && !!(serviceInfo && serviceInfo.nfv == 'yes')}
                                                            tooltip={""}
                                                            rounded
                                                        />
                                                    );
                                                })
                                            }
                                        </td>
                                        <td key={`${column_key}-planned`}>
                                            {nrenMap.has(column_key) &&
                                                showYears.map(year => {
                                                    const chargingYears = nrenMap.get(column_key)!;
                                                    const serviceInfo = chargingYears.get(year);
                                                    return (
                                                        <ColorPill
                                                            key={year}
                                                            year={year}
                                                            active={chargingYears.has(year) && !!(serviceInfo && serviceInfo.nfv == 'planned')}
                                                            tooltip={""}
                                                            rounded
                                                        />
                                                    );
                                                })
                                            }
                                        </td>
                                    </>
                                ))}

                                <td key={`${nren}-other-yes`}>
                                    {Array.from(nrenMap.keys()).filter(key => !["routers", "firewalls",
                                        "load_balancers", "vpn_concentrators"].includes(key)).map(other_key => (

                                            <div key={`${other_key}-yes`}>
                                                {nrenMap.has(other_key) &&
                                                    showYears.map(year => {
                                                        const chargingYears = nrenMap.get(other_key)!;
                                                        const serviceInfo = chargingYears.get(year);
                                                        return (
                                                            <ColorPill
                                                                key={year}
                                                                year={year}
                                                                active={chargingYears.has(year) && !!(serviceInfo && serviceInfo?.nfv == 'yes')}
                                                                tooltip={other_key}
                                                                rounded
                                                            />
                                                        );
                                                    })
                                                }
                                            </div>
                                        ))}
                                </td>
                                <td key={`${nren}-other-planned`}>
                                    {Array.from(nrenMap.keys()).filter(key => !["routers", "firewalls",
                                        "load_balancers", "vpn_concentrators"].includes(key)).map(other_key => (

                                            <div key={`${other_key}-planned`}>
                                                {nrenMap.has(other_key) &&
                                                    showYears.map(year => {
                                                        const chargingYears = nrenMap.get(other_key)!;
                                                        const serviceInfo = chargingYears.get(year);
                                                        return (
                                                            <ColorPill
                                                                key={year}
                                                                year={year}
                                                                active={chargingYears.has(year) && !!(serviceInfo && serviceInfo?.nfv == 'planned')}
                                                                tooltip={other_key}
                                                                rounded
                                                            />
                                                        );
                                                    })
                                                }
                                            </div>
                                        ))}
                                </td>
                            </tr>
                        ))}

                    </tbody>
                </Table>
            </ChartContainer>
        </DataPage>
    );
}
export default NetworkFunctionVirtualisationPage;
