import { useContext } from 'react';
import { Bar } from 'react-chartjs-2';
import { Chart as ChartJS, CategoryScale, LinearScale, BarElement, Title, Tooltip, Legend } from 'chart.js';

import { TrafficRatio } from "compendium/Schema";
import { createTrafficRatioDataset } from "compendium/helpers/dataconversion";
import DataPage from 'compendium/components/DataPage';
import Filter from "compendium/components/graphing/Filter"
import { Sections } from "compendium/helpers/constants";
import WithLegend from 'compendium/components/WithLegend';
import htmlLegendPlugin from 'compendium/plugins/HTMLLegendPlugin';
import { FilterSelectionContext } from 'compendium/providers/FilterSelectionProvider';
import { useData } from 'compendium/helpers/useData';
import titles from "compendium/titles";

ChartJS.register(
    CategoryScale,
    LinearScale,
    BarElement,
    Title,
    Tooltip,
    Legend
);

const chartOptions = {
    maintainAspectRatio: false,
    animation: {
        duration: 0
    },
    plugins: {
        htmlLegend: {
            // ID of the container(s) to put the legend in
            containerIDs: ['legendtop', 'legendbottom'],
        },
        legend: {
            display: false,
        },
        tooltip: {
            callbacks: {
                label: function (tooltipItem) {
                    let label = tooltipItem.dataset.label || '';

                    if (tooltipItem.parsed.x !== null) {
                        label += `: ${tooltipItem.parsed.x}%`
                    }
                    return label;
                }
            },

        },
    },
    scales: {
        x: {
            position: 'top' as const,
            stacked: true,
            ticks: {
                callback: (value: string | number, index: number) => {
                    return `${index * 10}%`;
                },
            },
        },
        x2: {
            ticks: {
                callback: (value: string | number) => {
                    if (typeof value === 'number') {
                        return `${value}%`;
                    }
                    return value;
                },
            },
            grid: {
                drawOnChartArea: false
            },
            afterDataLimits: function (axis) {
                const indices = Object.keys(ChartJS.instances)

                // initial values should be far outside possible range
                let max = -999999
                let min = 999999

                for (const index of indices) {
                    if (ChartJS.instances[index] && axis.chart.scales.x2) {
                        min = Math.min(ChartJS.instances[index].scales.x.min, min);
                        max = Math.max(ChartJS.instances[index].scales.x.max, max);
                    }
                }

                axis.chart.scales.x2.options.min = min;
                axis.chart.scales.x2.options.max = max;
                axis.chart.scales.x2.min = min;
                axis.chart.scales.x2.max = max;
            },
        },
        y: {
            stacked: true,
            ticks: {
                autoSkip: false,
            },
        },
    },
    indexAxis: "y" as const
};

function TrafficRatioPage() {
    const { filterSelection, setFilterSelection } = useContext(FilterSelectionContext);
    const { data: trafficRatioData, years, nrens } = useData<TrafficRatio>('/api/traffic-ratio', setFilterSelection);

    const selectedData = trafficRatioData.filter(data =>
        filterSelection.selectedYears.includes(data.year) && filterSelection.selectedNrens.includes(data.nren)
    );
    const trafficRatioDataset = createTrafficRatioDataset(selectedData, filterSelection.selectedYears[0]);

    const filterNode = <Filter
        max1year
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
    />

    const selectedNrens = Array.from(new Set(selectedData.map(data => data.nren))).map(nren => nrens.get(nren)).filter(nren => !!nren);
    const numNrens = selectedNrens.length;
    const heightPerBar = 1.5; // every added bar should give this much additional height

    // set a minimum height of 20rem
    const height = Math.max(numNrens * heightPerBar, 20);

    const title = titles['traffic-ratio'];
    const description = "The graph shows the ratio of commodity versus research and education traffic in NREN networks"

    const filename = "types_of_traffic_in_nren_networks";
    return (
        <DataPage title={title}
            description={description}
            category={Sections.Network} filter={filterNode}
            data={selectedData} filename={filename}>

            <WithLegend>
                <div className="chart-container" style={{ 'height': `${height}rem` }}>
                    <Bar
                        data={trafficRatioDataset}
                        options={chartOptions}
                        plugins={[htmlLegendPlugin]}
                    />
                </div>
            </WithLegend>
        </DataPage>
    );
}
export default TrafficRatioPage;
