import React, { useContext } from "react";
import { Table } from "react-bootstrap";

import { Service, ServiceCategory } from "compendium/Schema";
import { createNrenTableLookup } from "compendium/helpers/dataconversion";
import ColorBadgeService from "compendium/components/ColorBadgeService";
import DataPage from "compendium/components/DataPage";
import Filter from "compendium/components/graphing/Filter";
import { Sections } from "compendium/helpers/constants";
import { FilterSelectionContext } from "compendium/providers/FilterSelectionProvider";
import ChartContainer from "compendium/components/graphing/ChartContainer";
import { useData } from "compendium/helpers/useData";
import titles from "compendium/titles";

interface inputProps {
    category: ServiceCategory
}

function ServicesPage({ category }: inputProps): React.ReactElement {
    const { filterSelection, setFilterSelection } = useContext(FilterSelectionContext);
    const { data: serviceData, years, nrens } = useData<Service>('/api/nren-services', setFilterSelection);

    const latestYear = Math.max(...filterSelection.selectedYears);

    const selectedData = (serviceData).filter(data =>
        filterSelection.selectedYears.includes(data.year)
        && filterSelection.selectedNrens.includes(data.nren)
        && data.service_category == category
    );

    const servicesDict = {};
    selectedData.forEach(service => {
        servicesDict[service.service_name] = service.service_description;
    });
    const servicesList = Object.entries(servicesDict).sort((a, b) => a[0].toLowerCase() < b[0].toLowerCase() ? -1 : 1);

    const dataLookup = createNrenTableLookup(selectedData, 'service_name');

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
    />

    const showYears = [...filterSelection.selectedYears.filter(year => years.has(year))].sort();

    let title = '';

    switch (category) {
        case ServiceCategory.network_services:
            title = titles["network-services"];
            break;
        case ServiceCategory.isp_support:
            title = titles["isp-support-services"];
            break;
        case ServiceCategory.security:
            title = titles["security-services"];
            break;
        case ServiceCategory.identity:
            title = titles["identity-services"];
            break;
        case ServiceCategory.collaboration:
            title = titles["collaboration-services"];
            break;
        case ServiceCategory.multimedia:
            title = titles["multimedia-services"];
            break;
        case ServiceCategory.storage_and_hosting:
            title = titles["storage-and-hosting-services"];
            break;
        case ServiceCategory.professional_services:
            title = titles["professional-services"];
            break;
    }

    return (
        <DataPage title={title}
            description="The service matrix shows the services NRENs offer to their users. These 
            services are grouped thematically, with navigation possible via. the side menu. NRENs 
            are invited to give extra information about their services; where this is provided, 
            you will see a black circle around the marker. Hover over the marker to read more."
            category={Sections.Services} filter={filterNode}
            data={selectedData} filename="nren_services">
            <ChartContainer>
                <Table className="service-table" bordered>
                    <thead>
                        <tr>
                            <th></th>
                            {servicesList.map(([serviceName, serviceDescription], index) => (
                                <th key={serviceName} data-description={serviceDescription} className={`bottom-tooltip color-of-the-service-header-${index % 13}`}>{serviceName}</th>
                            ))}
                        </tr>
                    </thead>
                    <tbody>
                        {Array.from(dataLookup.entries()).map(([nren, nrenServiceMap]) => (
                            <tr key={nren}>
                                <td className="bold-text">{nren}</td>
                                {servicesList.map(([serviceName, _], index) => (
                                    <td key={serviceName}>
                                        {nrenServiceMap.has(serviceName) &&
                                            showYears.map(year => {
                                                const serviceYears = nrenServiceMap.get(serviceName)!;
                                                const serviceInfo = serviceYears.get(year);
                                                return <ColorBadgeService
                                                    key={year}
                                                    year={year}
                                                    active={serviceYears.has(year)}
                                                    serviceInfo={serviceInfo}
                                                    tickServiceIndex={index}
                                                    current={year == latestYear} />;
                                            })
                                        }
                                    </td>
                                ))}
                            </tr>
                        ))}
                    </tbody>
                </Table>
            </ChartContainer>
        </DataPage>
    );
}
export default ServicesPage;
