import React, { createContext, useState } from 'react';

const getConfigFromLocalStorage = () => {
    const storedConfig = JSON.parse(localStorage.getItem('config') ?? "{}");

    const config = {};
    for (const key in storedConfig) {
        const value = storedConfig[key];
        if (value.expireTime && value.expireTime < Date.now()) {
            continue;
        }
        if (!value) continue;
        config[key] = value;
    }
    return config;
}

const saveConfigToLocalStorage = (config) => {
    localStorage.setItem('config', JSON.stringify(config));
}

export type BaseConfig = {
    [K: string | number | symbol]: { [key: string]: string | number | boolean | Date | undefined | BaseConfig };
};

type ConfigContext<T extends BaseConfig> = {
    getConfig: (key: string) => T | undefined;
    setConfig: (key: string, value?: T, timeout?: Date) => void;
};

const configContext = createContext<ConfigContext<BaseConfig>>({
    getConfig: () => undefined,
    setConfig: () => { }
});

interface Props {
    children: React.ReactNode;
}

const ConfigProvider: React.FC<Props> = ({ children }) => {
    const [config, setConfig] = useState<BaseConfig>(getConfigFromLocalStorage());

    const updateConfig = (key, value?, timeout?: Date) => {
        if (!key) throw new Error('Valid config key must be provided');
        if (value == undefined) {
            const newConfig = { ...config };
            delete newConfig[key];
            setConfig(newConfig);
            saveConfigToLocalStorage(newConfig);
            return;
        }

        const newAsString = JSON.stringify(value);
        const existingAsString = JSON.stringify(config[key]?.value);
        if (newAsString === existingAsString) return;

        const expireTime = timeout ? timeout.getTime() : null;
        if (expireTime && expireTime < Date.now()) {
            throw new Error('Timeout must be in the future');
        }

        if (expireTime) {
            setConfig({ ...config, [key]: { value, expireTime } });
            saveConfigToLocalStorage({ ...config, [key]: { value, expireTime } });
        } else {
            setConfig({ ...config, [key]: { value } });
            saveConfigToLocalStorage({ ...config, [key]: { value } });
        }
    }

    const getConfig = (key) => {
        const value = config[key];
        if (value?.expireTime && value.expireTime < Date.now()) {
            // expired, remove from config
            updateConfig(key);
            return undefined;
        }
        if (value != undefined) return value.value;
        return undefined;
    }
    return (
        <configContext.Provider value={{ getConfig, setConfig: updateConfig }}>
            {children}
        </configContext.Provider>
    );
};

export { configContext };
export default ConfigProvider;