import { useEffect } from "react";
import { Question } from "survey-core";
import { Survey } from "survey-react-ui";
import { VerificationStatus } from 'compendium/survey/Schema';

function customDescriptionCallback(_, options) {
    // get the customDescription for matrix rows and set it in the title
    // attribute so that it shows up as a hover popup
    if (options.column['indexValue'] == 0 && 'item' in options.row) {
        const item = options.row['item'] as object;
        if (item['customDescription'] !== undefined) {
            options.htmlElement.parentElement?.children[0].children[0].setAttribute("description", item['customDescription']);
            options.htmlElement.parentElement?.children[0].children[0].classList.add("survey-tooltip");
        }
    }
}

function hideCheckboxLabels(_, options) {
    if (options.question.hideCheckboxLabels) {
        const classes = options.cssClasses;
        classes.root += " hidden-checkbox-labels";
    }
}

function fixTitleCss(_, options) {
    // in various cases the css we supplied to surveyModel.css gets ignored, so we check for those cases
    const selector = '[data-name="' + options.question.name + '"]';
    const header = document.querySelector(selector)?.querySelector('h5');
    if (header && !header.classList.contains('sv-header-flex')) {
        options.question.updateElementCss();  // apparently this is the call that's missing sometimes
    }
}

function setDescriptionHTML(_, options) {
    // WARNING: THIS ASSUMES THAT THE QUESTION DESCRIPTION IS NOT USER-PROVIDED.
    // SETTING THE HTML HERE IS A SECURITY RISK IF NOT PROPERLY SANITISED
    if (options.name !== "description") return;

    let text = options.text;

    if (!text.length) return;

    const shorthands = ["e.g.", "i.e.", "etc.", "vs."];

    for (const shorthand of shorthands) {
        if (text.includes(shorthand)) {
            text = text.replace(shorthand, shorthand.slice(0, -1)); // remove the period
        }
    }

    // because we split on ". " we need to handle sentences with shorthands in them
    const sentences = text.split(". ");

    for (let i = 0; i < sentences.length; i++) {
        if (sentences[i].length == 0) {
            continue;
        }
        for (const shorthand of shorthands) {
            if (sentences[i].includes(shorthand.slice(0, -1))) {
                sentences[i] = sentences[i].replace(shorthand.slice(0, -1), shorthand);
            }
        }
    }

    const makeListFromSentence = (sentence: string) => {
        if (!sentence.includes("*")) {
            if (!sentence.endsWith(".")) {
                return sentence + ".";
            }
            return sentence;
        }

        // if it includes * then make a list from it
        return sentence.split("*").map((s, i) => {
            if (i == 0) {
                return s;
            }
            if (i == 1) {
                return `<ul><li>${s}</li>`;
            }
            return `<li>${s}</li>`;
        }).join("") + "</ul>";

    }

    const html = sentences.map(s => s.length ? `<p>${makeListFromSentence(s)}</p>` : null).join("");
    options.html = html
}

function strikeThroughOrHide(question: Question) {
    const hasVisibleIf = !!question.visibleIf;

    const selector = '[data-name="' + question.name + '"]';
    const container = document.querySelector(selector) as HTMLElement;
    const header = container?.querySelector('h5');
    if (hasVisibleIf) {
        container.style.display = "none";
        return;
    }
    if (header) {
        header.style.textDecoration = "line-through";
    }
    // hide the question content
    const questionContent = document.querySelector(selector)?.querySelector('.sv-question__content') as HTMLElement;
    if (questionContent) {
        questionContent.style.display = "none";
    }

}

function setVerifyButton(question: Question, state: VerificationStatus, surveyModel) {

    surveyModel.verificationStatus.set(question.name, state);

    const btn = document.createElement("button");
    btn.type = "button";
    btn.className = "sv-action-bar-item verification";
    btn.innerHTML = state;

    if (state == VerificationStatus.Unverified) {
        btn.innerHTML = "No change from previous year";
        btn.className += " verification-required";
        btn.onclick = function () {
            if (surveyModel.mode == "display") {
                return;
            }
            question.validate();
            setVerifyButton(question, VerificationStatus.Verified, surveyModel);
        }
    } else {
        btn.innerHTML = "Answer updated"
        btn.className += " verification-ok";
    }

    const selector = '[data-name="' + question.name + '"]';
    const header = document.querySelector(selector)?.querySelector('h5');

    const old = header?.querySelector(".verification");
    if (old) {
        old.replaceWith(btn);
    } else {
        header?.appendChild(btn);
    }
}

function SurveyComponent({ surveyModel }) {
    
    useEffect(() => {
        const updateFromUnverified = (_, options) => {
            const currentStatus = surveyModel.verificationStatus.get(options.question.name);
            if (currentStatus == VerificationStatus.Unverified) {
                setVerifyButton(options.question, VerificationStatus.Edited, surveyModel);
            }
        }
        
        const alwaysSetVerify = (_, options) => {
            const status = surveyModel.verificationStatus.get(options.question.name);
            const readonly = options.question?.readOnly;
    
            if (status && !readonly) {
                setVerifyButton(options.question, status, surveyModel);
            }
            else if (readonly) {
                strikeThroughOrHide(options.question);
            }
        }

        if (!surveyModel.onAfterRenderQuestion.hasFunc(alwaysSetVerify)) {
            surveyModel.onAfterRenderQuestion.add(alwaysSetVerify);
            surveyModel.onAfterRenderQuestion.add(fixTitleCss);
        }

        if (!surveyModel.onValueChanged.hasFunc(updateFromUnverified)) {
            surveyModel.onValueChanged.add(updateFromUnverified);
        }

        if (!surveyModel.onUpdateQuestionCssClasses.hasFunc(hideCheckboxLabels)) {
            surveyModel.onUpdateQuestionCssClasses.add(hideCheckboxLabels);
        }

        if (!surveyModel.onMatrixAfterCellRender.hasFunc(customDescriptionCallback)) {
            // NB I would have preferred using onAfterRenderQuestion, but unfortunately that is
            // not always triggered on re-renders (specifically when extra columns become visble or invisible)
            surveyModel.onMatrixAfterCellRender.add(customDescriptionCallback);
        }

        if (!surveyModel.onTextMarkdown.hasFunc(setDescriptionHTML)) {
            // WARNING: THIS ASSUMES THAT THE QUESTION DESCRIPTION IS NOT USER-PROVIDED.
            // SETTING THE HTML HERE IS A SECURITY RISK IF NOT PROPERLY SANITISED
            surveyModel.onTextMarkdown.add(setDescriptionHTML);
        }
    }, [surveyModel]);

    return <Survey model={surveyModel} />
}

export default SurveyComponent;
