import { useState, useEffect, useRef } from "react";
import Accordion from 'react-bootstrap/Accordion';
import Button from 'react-bootstrap/Button';
import Row from 'react-bootstrap/Row';
import Table from 'react-bootstrap/Table';
import Container from "react-bootstrap/Container";
import toast, { Toaster } from "react-hot-toast";
import { SurveyStatus } from "compendium/survey/Schema";
import { fetchSurveys } from "compendium/survey/api/survey";
import { Survey } from "compendium/survey/api/types";
import SurveySidebar from "compendium/survey/management/SurveySidebar";
import { Spinner } from "react-bootstrap";
import StatusButton from "compendium/survey/StatusButton";
import { Link } from "react-router-dom";
import { debounce } from "lodash"
import { validateSurvey } from "compendium/survey/utils";

function updateSurveyNotes(year: number, nren_id: number, notes: string) {
    fetch('/api/survey/' + year + '/' + nren_id + '/notes', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({ notes: notes || '' })
    }).then(async response => {
        const body = await response.json();
        if (response.ok) {
            toast.success("Notes saved");
        } else {
            toast.error("Failed saving notes: " + body.message || response.statusText);
        }
    }).catch((error) => {
        toast.error("Failed saving notes: " + error);
    });

}

function ApiCallButton({ text, helpText, onClick, enabled }) {
    const [buttonLoading, setButtonLoading] = useState(false);

    const handleClick = async () => {
        if (buttonLoading) {
            return;
        }
        setButtonLoading(true);
        try {
            await onClick();
        } finally {
            setButtonLoading(false);
        }
    };

    return (
        <Button onClick={handleClick} disabled={!enabled} style={{ pointerEvents: 'auto', marginLeft: '.5rem' }} title={helpText}>
            {buttonLoading && <Spinner as="span" animation="border" size="sm" role="status" aria-hidden="true" />}
            {text}
        </Button>
    );
}

function SurveyManagementComponent() {
    const [surveys, setSurveys] = useState<Survey[]>([]);
    const surveyStatusUpdating = useRef(false);

    useEffect(() => {
        fetchSurveys().then((surveyList) => {
            setSurveys(surveyList);
        });
    }, []);

    async function apiCall(url: string, failureToast: string, successToast: string, dryRun: boolean = false) {
        try {
            if (dryRun) {
                url += "?dry_run=1";
            }
            const response = await fetch(url, { method: 'POST' });
            const json = await response.json();
            if (!response.ok) {
                toast(failureToast + json['message']);
            } else {
                if (json['message']) {
                    console.log(json['message']);
                }
                if (!dryRun) toast(successToast);
                fetchSurveys().then((surveyList) => {
                    setSurveys(surveyList);
                });
            }
        } catch (e) {
            toast(failureToast + (e as Error).message);
        }
    }

    async function newSurvey() {
        await apiCall('/api/survey/new', "Failed creating new survey: ", "Created new survey");
    }

    async function postSurveyStatus(year: number, status: string, dryRun: boolean = false) {
        if (surveyStatusUpdating.current) {
            toast("Wait for status update to be finished...");
            return;
        }
        surveyStatusUpdating.current = true;
        await apiCall(
            '/api/survey/' + status + '/' + year,
            "Error while updating " + year + " survey status to " + status + ": ",
            year + " survey status updated to " + status,
            dryRun
        );
        surveyStatusUpdating.current = false;
    }

    async function removeLock(year: number, nrenName: string) {
        await apiCall(
            '/api/response/unlock/' + year + '/' + nrenName,
            "Error while unlocking " + nrenName + " " + year + " survey response: ",
            nrenName + " " + year + " survey response unlocked"
        );
    }

    const newSurveyAllowed = surveys.length > 0 && surveys.every(s => s.status == SurveyStatus.published);

    const previewLink = window.location.origin + "/data?preview";

    return (
        <>
            <SurveySidebar />
            <Container className="py-5 grey-container">
                <Container style={{ maxWidth: '100rem' }}>
                    <Row >
                        <Toaster />
                        <Button onClick={newSurvey} disabled={!newSurveyAllowed} style={{ pointerEvents: 'auto', width: '10rem', margin: '1rem' }}
                            title="Create a new survey for the next year. Only possible if all current surveys are published.">
                            start new survey
                        </Button>
                        <Accordion defaultActiveKey="0">
                            {surveys.map((survey, index) => (
                                <Accordion.Item eventKey={index.toString()} key={survey.year} >
                                    <Accordion.Header>{survey.year} - {survey.status}</Accordion.Header>
                                    <Accordion.Body>
                                        <div style={{ marginLeft: '.5rem', marginBottom: '1rem' }}>
                                            <Link to={`/survey/admin/edit/${survey.year}`} target="_blank">
                                                <Button style={{ marginLeft: '.5rem' }}
                                                    title="Open the survey for inspection with all questions visible and any visibleIf logic added to the title.">
                                                    Inspect Survey
                                                </Button>
                                            </Link>
                                            <Link to={`/survey/admin/try/${survey.year}`} target="_blank">
                                                <Button style={{ marginLeft: '.5rem' }}
                                                    title="Open the survey exactly as the nrens will see it, but without any nren data.">
                                                    Try Survey
                                                </Button>
                                            </Link>
                                            {survey.status == SurveyStatus.closed &&
                                                <ApiCallButton text="Mark as open"
                                                    helpText="Allow the NRENs to respond to this survey. Only 1 survey may be open at a time, and (pre)-published surveys cannot be opened anymore."
                                                    enabled={survey.status == SurveyStatus.closed}
                                                    onClick={() => postSurveyStatus(survey.year, 'open')}
                                                />}
                                            {survey.status == SurveyStatus.open &&
                                                <ApiCallButton text="Mark as closed"
                                                    helpText="Do not allow the NRENs to respond to this survey anymore. Only surveys with status open can be closed."
                                                    enabled={survey.status == SurveyStatus.open}
                                                    onClick={() => postSurveyStatus(survey.year, 'close')}
                                                />}
                                            {(survey.status == SurveyStatus.preview || survey.status == SurveyStatus.published) &&
                                                <ApiCallButton text="Validate surveys"
                                                    helpText="Validate all survey responses. This will check if all required questions are answered and if the answers are in the correct format."
                                                    enabled={survey.status == SurveyStatus.preview || survey.status == SurveyStatus.published}
                                                    onClick={async () => {
                                                        let allValid = true;
                                                        const responses = [...survey.responses].sort((a, b) => a.nren.name.localeCompare(b.nren.name));
                                                        for (const response of responses) {
                                                            const valid = await validateSurvey(survey.year, response.nren.name);
                                                            if (!valid) {
                                                                toast.error(`${response.nren.name} ${survey.year} survey is not valid.`);
                                                            } else {
                                                                toast.success(`${response.nren.name} ${survey.year} survey is valid.`);
                                                            }
                                                            allValid = allValid && valid;   
                                                        }

                                                        if (!allValid) {
                                                            toast.error("Some surveys are not valid.\nPlease check the responses")
                                                        } else {
                                                            toast.success("All surveys are valid")
                                                        }

                                                        fetchSurveys().then((surveyList) => {
                                                            setSurveys(surveyList);
                                                        });
                                                    }}
                                                />}
                                            {(survey.status == SurveyStatus.closed || survey.status == SurveyStatus.preview) &&
                                                <ApiCallButton text="Preview results"
                                                    helpText="Publish all completed survey responses to the compendium website for preview by admins. This is only possible if the survey is closed or previewed already."
                                                    enabled={survey.status == SurveyStatus.closed || survey.status == SurveyStatus.preview}
                                                    onClick={() => postSurveyStatus(survey.year, 'preview')}
                                                />}
                                            {(survey.status == SurveyStatus.preview || survey.status == SurveyStatus.published) &&
                                                <>
                                                    <ApiCallButton text="Publish results (dry run)"
                                                        helpText="Performs a dry-run of the publish operation, without actually publishing the results. Changes are logged in the browser console (F12)."
                                                        enabled={survey.status == SurveyStatus.preview || survey.status == SurveyStatus.published}
                                                        onClick={() => postSurveyStatus(survey.year, 'publish', true)}
                                                    />
                                                    <ApiCallButton text="Publish results"
                                                        helpText="Publish or re-publish all completed survey responses to the compendium website. This is only possible if the survey is in preview or published already."
                                                        enabled={survey.status == SurveyStatus.preview || survey.status == SurveyStatus.published}
                                                        onClick={() => postSurveyStatus(survey.year, 'publish')}
                                                    />
                                                    {survey.status == SurveyStatus.preview && <span>&nbsp; Preview link: <a href={previewLink}>{previewLink}</a></span>}
                                                </>}
                                        </div>
                                        <Table>
                                            <colgroup>
                                                <col style={{ width: '10%' }} />
                                                <col style={{ width: '10%' }} />
                                                <col style={{ width: '10%' }} />
                                                <col style={{ width: '20%' }} />
                                                <col style={{ width: '30%' }} />
                                                <col style={{ width: '20%' }} />
                                            </colgroup>
                                            <thead>

                                                <tr>
                                                    <th>NREN</th>
                                                    <th>Response Status</th>
                                                    <th>Response Valid</th>
                                                    <th>Lock</th>
                                                    <th>Management Notes</th>
                                                    <th>Actions</th>
                                                </tr>
                                            </thead>
                                            <tbody>
                                                {survey.responses.map(response => (
                                                    <tr key={response.nren.id}>
                                                        <td>{response.nren.name}</td>
                                                        <td><StatusButton status={response.status} /></td>
                                                        <td><StatusButton status={response.valid ? "true" : "false"} /></td>
                                                        <td style={{ textWrap: "wrap", wordWrap: "break-word", maxWidth: "10rem" }}>{response.lock_description}</td>
                                                        <td>
                                                            {'notes' in response && <textarea onInput={debounce((e) => updateSurveyNotes(survey.year, response.nren.id, e.target.value), 1000)}
                                                                style={{ minWidth: '100%', minHeight: '5rem' }} placeholder="Notes for this survey" defaultValue={response.notes || ''}>
                                                            </textarea>}

                                                        </td>
                                                        <td>
                                                            <Link to={`/survey/response/${survey.year}/${response.nren.name}`} target="_blank">
                                                                <Button style={{ pointerEvents: 'auto', margin: '.5rem' }}
                                                                    title="Open the responses of the NREN.">
                                                                    open
                                                                </Button>
                                                            </Link>
                                                            <Button onClick={() => removeLock(survey.year, response.nren.name)} disabled={response.lock_description == ''} style={{ pointerEvents: 'auto' }}
                                                                title="Remove the lock from the survey so that another person can open the survey for editing. WARNING: The person that currently has the lock will not be able to save their changes anymore once someone else starts editing!">
                                                                remove lock
                                                            </Button>
                                                            {response.status == 'completed' &&
                                                                <Button onClick={() => {

                                                                    validateSurvey(survey.year, response.nren.name).then((isValid => {

                                                                        if (!isValid) {
                                                                            toast.error(`${response.nren.name} ${survey.year} survey is not valid.\nPlease check the response`)
                                                                        } else {
                                                                            toast.success("Survey is valid")
                                                                        }

                                                                        // Update the survey list, so that the valid status is updated
                                                                        fetchSurveys().then((surveyList) => {
                                                                            setSurveys(surveyList);
                                                                        });
                                                                    })).catch((error) => {
                                                                        toast.error("Failed validating survey: " + error);
                                                                    });
                                                                }
                                                                } style={{ pointerEvents: 'auto' }}>
                                                                    Validate Survey
                                                                </Button>}
                                                        </td>
                                                    </tr>
                                                ))}

                                            </tbody>
                                        </Table>

                                    </Accordion.Body>
                                </Accordion.Item>
                            ))}
                        </Accordion>
                    </Row>
                </Container>
            </Container>
        </>
    );

}

export default SurveyManagementComponent;
