"""
This module handles adding responses from the 2023 survey into the presentation_models.
The process is straightforward as the data is new and can be directly inserted into the presentation_models.

For subsequent years like 2024, handling should be almost the same as 2023, only changes in questions
will need to be addressed.

Usage:
    Used in publish_survey API in compendium_v2/routes/survey.py
"""

import json
import logging

from typing import Sequence, Dict, Any
from sqlalchemy import select, delete
from enum import Enum
from decimal import Decimal

from compendium_v2.db import db

from compendium_v2.db.survey_models import ResponseStatus, SurveyResponse
from compendium_v2.publishers.year.map_2023 import map_2023
from compendium_v2.publishers.year.map_2024 import map_2024
from compendium_v2.db.presentation_models import (
    NRENExclusion,
    PresentationModel,
    AlienWave,
    BudgetEntry,
    Capacity,
    CentralProcurement,
    CertificateProviders,
    ChargingStructure,
    CommercialChargingLevel,
    CommercialConnectivity,
    ConnectedProportion,
    ConnectionCarrier,
    ConnectivityGrowth,
    ConnectivityLevel,
    ConnectivityLoad,
    CrisisExercises,
    DarkFibreInstalled,
    DarkFibreLease,
    ECProject,
    EOSCListings,
    ExternalConnections,
    FibreLight,
    FundingSource,
    InstitutionURLs,
    MonitoringTools,
    NetworkAutomation,
    NetworkFunctionVirtualisation,
    NetworkMapUrls,
    NonREPeers,
    NREN,
    NRENService,
    NrenStaff,
    OpsAutomation,
    ParentOrganization,
    PassiveMonitoring,
    PertTeam,
    Policy,
    RemoteCampuses,
    SecurityControls,
    ServiceManagement,
    ServiceUserTypes,
    SiemVendors,
    Standards,
    SubOrganization,
    TrafficRatio,
    TrafficStatistics,
    TrafficVolume,
    WeatherMap,
)
from compendium_v2.publishers.utils import redirect_stdout

logger = logging.getLogger(__name__)


def do_comparison(data, year):
    models = [
        AlienWave,
        BudgetEntry,
        Capacity,
        CentralProcurement,
        CertificateProviders,
        ChargingStructure,
        CommercialChargingLevel,
        CommercialConnectivity,
        ConnectedProportion,
        ConnectionCarrier,
        ConnectivityGrowth,
        ConnectivityLevel,
        ConnectivityLoad,
        CrisisExercises,
        DarkFibreInstalled,
        DarkFibreLease,
        ECProject,
        EOSCListings,
        ExternalConnections,
        FibreLight,
        FundingSource,
        InstitutionURLs,
        MonitoringTools,
        NetworkAutomation,
        NetworkFunctionVirtualisation,
        NetworkMapUrls,
        NonREPeers,
        NREN,
        NRENService,
        NrenStaff,
        OpsAutomation,
        ParentOrganization,
        PassiveMonitoring,
        PertTeam,
        Policy,
        RemoteCampuses,
        SecurityControls,
        ServiceManagement,
        ServiceUserTypes,
        SiemVendors,
        Standards,
        SubOrganization,
        TrafficRatio,
        TrafficStatistics,
        TrafficVolume,
        WeatherMap
    ]

    all_models = [model for model in models if model.__name__ not in ['NREN', 'NRENService', 'Service']]

    nrens = {nren.id: nren for nren in db.session.scalars(select(NREN)).unique().all()}

    def convert_to_simple(row):
        if isinstance(row, dict):
            new_dict = {}
            for key, val in row.items():
                new_dict[key] = convert_to_simple(val)
            return new_dict

        if isinstance(row, list):
            return [convert_to_simple(v) for v in row]

        # convert to simple types for comparison
        if isinstance(row, Enum):
            return row.value
        if isinstance(row, Decimal):
            return float(row)
        return row

    def get_difference(existing, new):
        """
        This function compares the existing data with the new data and returns the differences.
        Whitespace changes are ignored.
        """

        equal = True

        def _not_equal(v1, v2):
            nonlocal equal

            if isinstance(v1, str):
                v1 = v1.strip()
            if isinstance(v2, str):
                v2 = v2.strip()
            both_equal = v1 == v2
            equal = equal and both_equal
            return not both_equal

        has_user_category = 'user_category' in row
        has_service_key = 'service_key' in row
        has_service_category = 'service_category' in row

        if has_service_category:
            # has both service_category and user_category, we need to check both
            differences = {k: (existing.get(k), '->', v) for k, v in new.items() if _not_equal(existing.get(k), v)
                           and existing.get('user_category') == new['user_category']
                           and existing.get('service_category') == new['service_category']
                           }
        elif has_user_category:
            differences = {k: (existing.get(k), '->', v) for k, v in new.items() if _not_equal(existing.get(k), v)
                           and existing.get('user_category') == new['user_category']}
        elif has_service_key:
            differences = {k: (existing.get(k), '->', v) for k, v in new.items()
                           if _not_equal(existing.get(k), v) and existing.get('service_key') == new['service_key']}
        else:
            differences = {k: (existing.get(k), '->', v) for k, v in new.items() if _not_equal(existing.get(k), v)}

        return equal, differences

    # compare existing data for year with new data
    for model, rows in list(data.items()):
        if model not in models:
            raise ValueError(f"Model {model.__name__} not found in list of models for comparison, new model?")
        all_models = [m for m in all_models if m.__name__ != model.__name__]
        not_found = []
        existing_data = db.session.scalars(select(model).where(model.year == year)).unique().all()
        existing_dicts = []

        for row in existing_data:
            if hasattr(row, 'compare_dict'):
                values = row.compare_dict()
            else:
                values = row.__dict__.copy()
            for key, value in list(values.items()):
                if key in ['_sa_instance_state', 'nren', 'nren_country']:
                    values.pop(key)
                    continue
                if isinstance(value, db.Model):
                    values.pop(key)

            values = convert_to_simple(values)
            existing_dicts.append(values)

        if existing_data:
            # compare existing data with new data
            for row in rows:
                row = convert_to_simple(row)
                nren_id = row['nren_id']
                has_user_category = 'user_category' in row
                has_service_key = 'service_key' in row
                has_service_category = 'service_category' in row
                has_organization = 'organization' in row

                if has_organization:
                    existing = next(
                        (r for r in existing_dicts if r['nren_id'] == nren_id
                         and r['organization'].strip() == row['organization'].strip()), None)
                elif has_service_category:
                    existing = next(
                        (r for r in existing_dicts if r['nren_id'] == nren_id
                         and r['user_category'] == row['user_category']
                         and r['service_category'] == row['service_category']), None)
                elif has_user_category:
                    existing = next(
                        (r for r in existing_dicts if r['nren_id'] == nren_id
                         and r['user_category'] == row['user_category']), None)
                elif has_service_key:
                    existing = next(
                        (r for r in existing_dicts if r['nren_id'] == nren_id
                         and r['service_key'] == row['service_key']), None)
                elif model in [ECProject]:
                    # workaround for ecprojects not being a list, but individual instances per value
                    existing = next((r for r in existing_dicts if r['nren_id'] == nren_id
                                     and r['project'] == row['project']), None)
                else:
                    existing = next((r for r in existing_dicts if r['nren_id'] == nren_id), None)

                if row == existing:
                    existing_dicts.remove(existing)
                    continue

                if existing is None:
                    not_found.append(row)
                    continue

                equal, differences = get_difference(existing, row)

                if differences:
                    print(f"{model.__name__} is different for NREN {nrens[nren_id].name} ({year}): {differences}")
                if equal or differences:
                    existing_dicts.remove(existing)

        if not_found:
            print("====================================================")
            print(f"New data found for {model.__name__} for year {year}")
            print(json.dumps(not_found, indent=2))
            print("====================================================")
        if existing_dicts:
            print("====================================================")
            print(f"Data missing for {model.__name__} for year {year}")
            print(json.dumps(existing_dicts, indent=2))
            print("====================================================")

        if not not_found and not existing_dicts:
            print(f"Data matches for {model.__name__} for year {year}")

    for _model in all_models:
        has_year_data = db.session.scalars(select(_model).where(_model.year == year)).unique().all()
        if has_year_data:
            print(f"Data not processed for {_model.__name__} for year {year}")


def save_data(year: int, data: Dict[PresentationModel, Sequence[Dict[str, Any]]]):
    table_classes = list(data.keys())
    if not all(issubclass(table, db.Model) for table in table_classes):
        raise ValueError("All tables must be subclasses of db.Model")

    for table_class in table_classes:
        db.session.execute(delete(table_class).where(table_class.year == year))
    for model, rows in data.items():
        db.session.bulk_insert_mappings(model, rows)  # type: ignore
        db.session.commit()


def publish(year, dry_run=False):
    responses = db.session.scalars(
        select(SurveyResponse).where(
            SurveyResponse.survey_year == year, SurveyResponse.status == ResponseStatus.completed,
            SurveyResponse.valid == True).where(~SurveyResponse.nren_id.in_(  # noqa: E712
                select(NRENExclusion.nren_id).where(NRENExclusion.year == year)
            )
        )
    ).unique().all()

    question_mapping = {
        2016: lambda r: map_2024(r, 2016),
        2017: lambda r: map_2024(r, 2017),
        2018: lambda r: map_2024(r, 2018),
        2019: lambda r: map_2024(r, 2019),
        2020: lambda r: map_2024(r, 2020),
        2021: lambda r: map_2024(r, 2021),
        2022: lambda r: map_2024(r, 2022),
        2023: map_2023,
        2024: map_2024
    }

    mapping_function = question_mapping.get(year)

    if not mapping_function:
        raise ValueError(f"No publishing function found for the {year} survey.")

    data = mapping_function(responses)

    with redirect_stdout() as output:
        do_comparison(data, year)
    output = output.getvalue()
    if not dry_run:
        save_data(year, data)
        output += f"\nPublished data for {year} survey"
    logger.info('\n'+output)
    return output
